# The name of the kmod package here.
%define kmod_name kpatch
%define module %{kmod_name}.ko

# Get the newest available version of vzkernel from vzkernel-devel.
# Added the script to sources here for _{SOURCE42} to be defined.
Source42:	get-kernel-version.sh
%define kversion_newest %(sh %{SOURCE42})
%{!?kernel_version: %define kernel_version %{kversion_newest}}
%define vzver %(echo '%{kernel_version}' | sed -e 's/^.*vz7\\.//')

# These defines are convenient to make the spec files for VZ7 and VL7 as
# close as possible.
%define kversion_base %{kernel_version}
%define kversion_dist %{kversion_base}
%define kversion %{kversion_dist}

%define gitrev f80c2cf
%define gitdate 20160301

# How to prepare the source archive from the git tree:
# $ hash=`git log -1 --format='%h'`
# $ date=`git log -1 --format='%cd' --date=short | tr -d -`
# $ git archive --prefix kpatch-${date}-git${hash}/ ${hash} \
#       | xz -7e > kpatch-${date}-git${hash}.tar.xz

%global _use_internal_dependency_generator 0

# Disable the building of the debug package(s).
%define debug_package %{nil}

# Do not strip files automatically, this will strip the module's signature
# as well
%define __strip /usr/bin/true

# The suffix of the subpackage's name.
%define subpkg %{vzver}


Name:		%{kmod_name}-kmod
Version:	0.3.3
Release:	90.%{gitdate}%{?dist}
Summary:	Dynamic kernel patching tools - the kernel-mode part

Group:		System Environment/Kernel
License:	GPLv2
URL:		https://github.com/dynup/kpatch

Source0:	%{kmod_name}-%{gitdate}-git%{gitrev}.tar.xz

# The patches from upstream.
#Patch0:

# VZ-specific patches
Patch501:	vz001-kmod-core-Do-not-taint-the-kernel.patch
Patch502:	vz002-kmod-core-Skip-relocations-of-already-altered-instr.patch
Patch503:	vz003-kmod-core-added-a-minimal-fault-simulation-support.patch
Patch504:	vz004-Handle-relocations-of-the-functions-with-same-names.patch
Patch505:	vz005-Allow-for-extension-of-Kpatch-related-structures.patch

ExclusiveArch:	x86_64

BuildRequires:	vzkernel-devel
BuildRequires:	perl
BuildRequires:	openssl
BuildRequires:	module-signing-keys

%description
This package contains %{module} kernel module, the kernel-mode part of
Kpatch. Kpatch is a Linux dynamic kernel patching tool which allows you
to patch a running kernel without rebooting or restarting any processes.

#===========================================================================

%package %{subpkg}

Summary:	Dynamic kernel patch manager - the kernel-mode part
Group:		System Environment/Kernel
License:	GPLv2

# Do not generate requirements automatically. By default the requirements
# like "kernel(__kmalloc) = abcd1234" are generated and tie the package to
# the corresponding kernel versions.
# kpatch-patch should pull the right kpatch-kmod package, so it is not
# needed to require the particular kernel explicitly. Such requirements
# proved to be problematic.
AutoReq: no

Provides:	kernel-modules >= %{kversion_dist}.%{_target_cpu}
# kpatch-build requires kpatch-kmod >= <...>
Provides:	%{kmod_name}-kmod = %{?epoch:%{epoch}:}%{version}-%{release}
# Just in case someone looks for kmod-kpatch-<...>:
Provides:	kmod-%{kmod_name}-%{kversion_base} = %{?epoch:%{epoch}:}%{version}-%{release}
# kpatch-patch requires kpatch-kmod-<kernel_version>
Provides:	%{kmod_name}-kmod-%{kversion_base} = %{?epoch:%{epoch}:}%{version}-%{release}

Requires(post):		/usr/sbin/depmod
Requires(post):		kmod
Requires(postun):	/usr/sbin/depmod
Requires(postun):	kmod

# The plugin will automatically remove unneeded kpatch-kmod* and
# readykernel-patch* packages.
Requires:	yum-plugin-readykernel


%description %{subpkg}
This package contains %{module} kernel module, the kernel-mode part of
Kpatch. Kpatch is a Linux dynamic kernel patching tool which allows you
to patch a running kernel without rebooting or restarting any processes.


%files %{subpkg}
%defattr(644,root,root,755)
%dir /lib/modules/%{kversion}
%dir /lib/modules/%{kversion}/extra
%dir /lib/modules/%{kversion}/extra/%{kmod_name}
/lib/modules/%{kversion}/extra/%{kmod_name}/*


%post %{subpkg}
echo "Running depmod. This may take some time ..."
if [ -e "/boot/System.map-%{kversion}" ]; then
    /usr/sbin/depmod -aeF "/boot/System.map-%{kversion}" "%{kversion}" > /dev/null || :
fi
echo "Done."


%postun %{subpkg}
echo "Running depmod. This may take some time ..."
if [ -e "/boot/System.map-%{kversion}" ]; then
    /usr/sbin/depmod -aeF "/boot/System.map-%{kversion}" "%{kversion}" > /dev/null || :
fi
echo "Done."

#===========================================================================

%prep
%setup -qn %{kmod_name}-%{gitdate}-git%{gitrev}
#patch0 -p1
%patch501 -p1
%patch502 -p1
%patch503 -p1
%patch504 -p1
%patch505 -p1


%build
echo "Building Kpatch core module for the kernel %{kversion}."
make KPATCH_BUILD=/usr/src/kernels/%{kversion}/ -C kmod/core %{_smp_mflags}


%install
rm -rf %{buildroot}

install -d %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/
install -m 644 kmod/core/%{module} %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/
install -m 644 kmod/core/Module.symvers %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/

# Strip debug info from the module.
# Do not use _{__strip} macro here because it is redefined above.
/usr/bin/strip --strip-debug %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/%{module}

# Sign the module if the keys are available.
KEYPATH=%{_datadir}/module-signing-keys/kpatch/
MODSECKEY="${KEYPATH}/signing_key_kpatch.priv"
MODPUBKEY="${KEYPATH}/signing_key_kpatch.x509"
MODULE_FILE="%{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/%{module}"

if [[ -f "${MODSECKEY}" ]]; then
	/usr/src/kernels/%{kversion}/scripts/sign-file sha256 \
	  ${MODSECKEY} ${MODPUBKEY} ${MODULE_FILE}
	rm -f ${MODULE_FILE}.{sig,dig}

	if [ "~Module signature appended~" != "$(tail -c 28 ${MODULE_FILE})" ]; then
		echo "Module %{module} is still unsigned."
		exit 1
	fi
fi
#===========================================================================


%changelog
* Fri Jul 8 2016 Evgenii Shatokhin <eshatokhin@virtuozzo.com> 0.3.3-56.20160301
- handle relocations of the functions with identical names (PSBM-49192)
- allow for extension of Kpatch-related structures in the future

* Mon Jul 4 2016 Evgenii Shatokhin <eshatokhin@virtuozzo.com> 0.3.3-53.20160301
- make sure the package owns the appropriate directoroies

* Tue Mar 29 2016 Evgenii Shatokhin <eshatokhin@virtuozzo.com> 0.3.2-1.20160301.2
- added a workaround for https://github.com/dynup/kpatch/issues/580

* Mon Mar 14 2016 Evgenii Shatokhin <eshatokhin@virtuozzo.com> 0.3.2-1.20160301.1
- update to revision f80c2cf from the upstream git (v.0.3.2+)
- build for the kernel 3.10.0-327.10.1.vz7.12.1

* Fri Feb 5 2016 Evgenii Shatokhin <eshatokhin@virtuozzo.com> 0.3.1-0.20151118.4
- build for the kernel 3.10.0-327.3.1.vz7.10.10

* Tue Jan 12 2016 Evgenii Shatokhin <eshatokhin@odin.com> 0.3.1-0.20151118.4
- unify the spec files for VZ7 and VL7 as much as possible
- build for the kernel 3.10.0-229.7.2.vz7.9.20

* Mon Nov 30 2015 Evgenii Shatokhin <eshatokhin@odin.com> 0.3.1-0.20151118.1
- build for the kernel 3.10.0-229.7.2.vz7.9.14

* Tue Nov 24 2015 Evgenii Shatokhin <eshatokhin@odin.com> 0.3.1-0.20151118.1
- update to revision b60d3ac from the upstream git (v.0.3.1+)

* Wed Nov 11 2015 Evgenii Shatokhin <eshatokhin@odin.com> 0.2.2-0.20151104.1
- update to revision eb54876 from the upstream git

* Mon Oct 26 2015 Evgenii Shatokhin <eshatokhin@odin.com> 0.2.2-0.20151021.1
- update to revision ff40e54 from the upstream git

* Mon Oct 19 2015 Evgenii Shatokhin <eshatokhin@odin.com> 0.2.2-0.20151016.1
- update to revision d444caa from the upstream git

* Thu Oct 8 2015 Evgenii Shatokhin <eshatokhin@odin.com> 0.2.2-0.20151001.1
- Initial build (based on revision f049dcf from the upstream git)
